/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.common;

import com.google.common.testing.EqualsTester;
import org.junit.Test;

import java.util.regex.Pattern;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.onosproject.nemo.model.common.Uuid.E_BAD_FORMAT;
import static org.onosproject.nemo.model.common.Uuid.UUID_PATTERN;

/**
 * Unit tests for {@link Uuid}.
 */
public class UuidTest extends AbstractCommonTest {

    private static final String ID = "14ce424a-3e50-4a2a-ad5c-b29845158c8b";
    private static final String OTHER_ID = "a4ce424a-3e50-4a2a-ad5c-b29845158c8b";

    private static final String[] BAD_STRINGS = {
            "%..",
            "123",
            "s4ce424a-3e50-4a2a-ad5c-b29845158c8b",
            "s4ce424a-3e50-4a2a-ad5c-b29845158c8basdf----"
    };
    private static final String[] GOOD_STRINGS = {
            "14ce424a-3e50-4a2a-ad5c-b29845158c8b",
            "a4ce424a-3e50-4a2a-bd5c-b29845158c8c",
            "F3AC116A-4E82-4A2A-AD5C-B29845158C8B"
    };

    private static final Pattern RE_UUID = Pattern.compile(UUID_PATTERN);

    private Uuid id;
    private Uuid copy;
    private Uuid diff;

    @Test
    public void fromString() {
        id = new Uuid(ID);
        assertEquals("id not set", ID, id.getValue());
    }

    @Test
    public void copyConstructor() {
        id = new Uuid(ID);
        copy = new Uuid(id);
        new EqualsTester()
                .addEqualityGroup(id, copy)
                .testEquals();
    }

    @Test(expected = NullPointerException.class)
    public void constructWithNullString() {
        new Uuid((String) null);
    }

    @Test(expected = NullPointerException.class)
    public void copyConstructWithNull() {
        new Uuid((Uuid) null);
    }

    @Test
    public void idNotMatch() {
        for (String s : BAD_STRINGS) {
            try {
                new Uuid(s);
                fail(E_NOEX + COLON + s);
            } catch (IllegalArgumentException e) {
                assertEquals(E_WREX, String.format(E_BAD_FORMAT, s, UUID_PATTERN),
                        e.getMessage());
            }
        }
    }

    @Test
    public void idMatch() {
        for (String s : GOOD_STRINGS) {
            try {
                new Uuid(s);
            } catch (IllegalArgumentException e) {
                fail(E_UNEX + COLON + s);
            }
        }
    }

    @Test
    public void equality() {
        id = new Uuid(ID);
        copy = new Uuid(ID);
        diff = new Uuid(OTHER_ID);
        new EqualsTester()
                .addEqualityGroup(id, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }
}